@@ -58,6 +58,18 @@ a ``True`` value. Integers and floats are handled exactly as you'd expect::
     >>> print record.INFO['AF']
     [0.5]
 
+There are a number of convienience functions for each ``Record`` allowing you to
+examine properties of interest::
+
+    >>> print record.num_called, record.call_rate, record.num_unknown
+    3 1.0 0
+    >>> print record.num_hom_ref, record.num_het, record.num_hom_alt
+    1 1 1
+    >>> print record.nucl_diversity, record.aaf
+    0.6 0.5
+    >>> print record.get_hets()
+    [Call(sample=NA00002, GT=1|0)]
+
 ``record.FORMAT`` will be a string specifying the format of the genotype
 fields.  In case the FORMAT column does not exist, ``record.FORMAT`` is
 ``None``.  Finally, ``record.samples`` is a list of dictionaries containing the
@@ -73,6 +85,24 @@ by sample name::
     >>> print record.genotype('NA00001')['GT']
     0|0
 
+The genotypes are represented by ``Call`` objects, which have three attributes: the
+corresponding Record ``site``, the sample name in ``sample`` and a dictionary of
+call data in ``data``::
+
+     >>> call = record.genotype('NA00001')
+     >>> print call.site
+     Record(CHROM=20, POS=17330, REF=T, ALT=['A'])
+     >>> print call.sample
+     NA00001
+     >>> print call.data
+     {'GT': '0|0', 'HQ': [58, 50], 'DP': [3], 'GQ': [49]}
+
+There are also a number of methods::
+
+    >>> print call.called, call.gt_type, call.gt_bases, call.phased
+    True 0 T|T True
+
+
 Metadata regarding the VCF file itself can be investigated through the
 following attributes:
 
@@ -112,6 +142,7 @@ import collections
 import re
 import csv
 import gzip
+import sys
 import itertools
 
 
@@ -219,9 +250,85 @@ class _vcf_metadata_parser(object):
         return match.group('key'), match.group('val')
 
 
+class _Call(object):
+    """ A called genotype, an entry in a VCF file"""
+
+
+    def __init__(self, site, sample, data):
+        self.site = site
+        self.sample = sample
+        self.data = data
+        self.gt_nums = self.data['GT']
+        self.called = self.gt_nums is not None and self.gt_nums != "./."
+
+    def __repr__(self):
+        return "Call(sample=%s, GT=%s)" % (self.sample, self.gt_nums)
+
+    def __eq__(self, other):
+        return (self.sample == other.sample and self.data == other.data)
+
+    @property
+    def gt_bases(self):
+        '''Return the actual genotype alleles.
+           E.g. if VCF genotype is 0/1, return A/G
+        '''
+        # nothing to do if no genotype call
+        if self.called:
+            # grab the numeric alleles of the gt string; tokenize by phasing
+            phase_char = "/" if not self.phased else "|"
+            (a1, a2) = self.gt_nums.split(phase_char)
+            # lookup and return the actual DNA alleles
+            try:
+                return self.site.alleles[int(a1)] + \
+                       phase_char + \
+                       self.site.alleles[int(a2)]
+            except:
+                sys.stderr.write("Allele number not found in list of alleles\n")
+        else:
+            return None
+
+    @property
+    def gt_type(self):
+        '''Return the type of genotype.
+           hom_ref  = 0
+           het      = 1
+           hom_alt  = 2  (we don;t track _which+ ALT)
+           uncalled = None
+        '''
+        # extract the numeric alleles of the gt string
+        if self.called:
+            # grab the numeric alleles of the gt string; tokenize by phasing
+            (a1, a2) = self.gt_nums.split("/") \
+                if not self.phased else self.gt_nums.split("|")
+            # infer genotype type from allele numbers
+            if (int(a1) == 0) and (int(a2) == 0): return 0
+            elif (int(a1) == 0) and (int(a2) >= 1): return 1
+            elif (int(a2) == 0) and (int(a1) >= 1): return 1
+            elif (int(a1) >= 1) and (int(a2) >= 1):
+                # same alt, so hom_alt
+                if a1 == a2: return 2
+                # diff alts, so het
+                else: return 1
+            else: return -1
+        else:
+            return None
+
+    @property
+    def phased(self):
+        '''Return a boolean indicating whether or not
+           the genotype is phased for this sample
+        '''
+        return self.data['GT'].find("|") >= 0
+
+    def __getitem__(self, key):
+        """ Lookup value, backwards compatibility """
+        return self.data[key]
+
 
 class _Record(object):
-    def __init__(self, CHROM, POS, ID, REF, ALT, QUAL, FILTER, INFO, FORMAT, samples, sample_indexes):
+    """ A set of calls at a site.  A row in a VCF file. """
+
+    def __init__(self, CHROM, POS, ID, REF, ALT, QUAL, FILTER, INFO, FORMAT, sample_indexes, samples=None):
         self.CHROM = CHROM
         self.POS = POS
         self.ID = ID
@@ -231,6 +338,9 @@ class _Record(object):
         self.FILTER = FILTER
         self.INFO = INFO
         self.FORMAT = FORMAT
+        # create a list of alleles. [0] = REF, [1:] = ALTS
+        self.alleles = [self.REF]
+        self.alleles.extend(self.ALT)
         self.samples = samples
         self._sample_indexes = sample_indexes
 
@@ -250,10 +360,90 @@ class _Record(object):
     def add_info(self, info, value=True):
         self.INFO[info] = value
 
-
     def genotype(self, name):
         return self.samples[self._sample_indexes[name]]
 
+    @property
+    def num_called(self):
+        """Return the number of called samples"""
+        return sum(s.called for s in self.samples)
+
+    @property
+    def call_rate(self):
+        """ return the fraction of genotypes that were actually called. """
+        return float(self.num_called) / float(len(self.samples))
+
+    @property
+    def num_hom_ref(self):
+        """ return the number of homozygous for ref allele genotypes"""
+        return len([s for s in self.samples if s.gt_type == 0])
+
+    @property
+    def num_hom_alt(self):
+        """ return the number of homozygous for alt allele genotypes"""
+        return len([s for s in self.samples if s.gt_type == 2])
+
+    @property
+    def num_het(self):
+        """ return the number of heterozygous genotypes"""
+        return len([s for s in self.samples if s.gt_type == 1])
+
+    @property
+    def num_unknown(self):
+        """ return the number of unknown genotypes"""
+        return len([s for s in self.samples if s.gt_type is None])
+
+    @property
+    def aaf(self):
+        """Calculate the allele frequency of the alternate allele.
+           NOTE 1: Punt if more than one alternate allele.
+           NOTE 2: Denominator calc'ed from _called_ genotypes.
+        """
+        # skip if more than one alternate allele. assumes bi-allelic
+        if len(self.ALT) > 1:
+            return None
+        hom_ref = self.num_hom_ref
+        het = self.num_het
+        hom_alt = self.num_hom_alt
+        num_chroms = float(2.0*self.num_called)
+        return float(het + 2*hom_alt)/float(num_chroms)
+
+    @property
+    def nucl_diversity(self):
+        """
+        Calculate pi_hat (estimation of nucleotide diversity) for the site.
+        This metric can be summed across multiple sites to compute regional
+        nucleotide diversity estimates.  For example, pi_hat for all variants
+        in a given gene.
+
+        Derived from:
+        \"Population Genetics: A Concise Guide, 2nd ed., p.45\"
+          John Gillespie.
+        """
+        # skip if more than one alternate allele. assumes bi-allelic
+        if len(self.ALT) > 1:
+            return None
+        p = self.aaf
+        q = 1.0-p
+        num_chroms = float(2.0*self.num_called)
+        return float(num_chroms/(num_chroms-1.0)) * (2.0 * p * q)
+
+    def get_hom_refs(self):
+        """ return the list of hom ref genotypes"""
+        return [s for s in self.samples if s.gt_type == 0]
+
+    def get_hom_alts(self):
+        """ return the list of hom alt genotypes"""
+        return [s for s in self.samples if s.gt_type == 2]
+
+    def get_hets(self):
+        """ return the list of het genotypes"""
+        return [s for s in self.samples if s.gt_type == 1]
+
+    def get_unknowns(self):
+        """ return the list of unknown genotypes"""
+        return [s for s in self.samples if s.gt_type is None]
+
 
 class Reader(object):
     '''Read and parse a VCF v 4.0 file'''
@@ -385,39 +575,57 @@ class Reader(object):
 
         return retdict
 
-    def _parse_samples(self, samples, samp_fmt):
+    def _parse_samples(self, samples, samp_fmt, site):
         '''Parse a sample entry according to the format specified in the FORMAT
         column.'''
         samp_data = []# OrderedDict()
         samp_fmt = samp_fmt.split(':')
 
-        # cache ref to speed loop
-        formats = self.formats
-        mapper = self._mapper
+        samp_fmt_types = []
+
+        for fmt in samp_fmt:
+            try:
+                entry_type = self.formats[fmt].type
+            except KeyError:
+                try:
+                    entry_type = RESERVED_FORMAT[fmt]
+                except KeyError:
+                    entry_type = 'String'
+            samp_fmt_types.append(entry_type)
 
         for name, sample in itertools.izip(self.samples, samples):
-            sampdict = dict(itertools.izip(samp_fmt, sample.split(':')))
-            for fmt in sampdict:
-                vals = sampdict[fmt].split(',')
-                try:
-                    entry_type = formats[fmt].type
-                except KeyError:
-                    try:
-                        entry_type = RESERVED_FORMAT[fmt]
-                    except KeyError:
-                        entry_type = 'String'
+            sampdict = self._parse_sample(sample, samp_fmt, samp_fmt_types)
+            samp_data.append(_Call(site, name, sampdict))
 
+        return samp_data
+
+    def _parse_sample(self, sample, samp_fmt, samp_fmt_types):
+        mapper = self._mapper
+
+        sampdict = dict([(x, None) for x in samp_fmt])
+
+        for fmt, entry_type, vals in itertools.izip(samp_fmt, samp_fmt_types, sample.split(':')):
+            vals = vals.split(',')
+
+            if fmt == 'GT':
+
+                gt = vals[0]
+
+                if gt == './.':
+                    if self.aggro:
+                        gt = None
+                    sampdict[fmt] = gt
+                    break
+                else:
+                    sampdict[fmt] = gt
+            else:
                 if entry_type == 'Integer':
                     sampdict[fmt] = mapper(int, vals)
                 elif entry_type == 'Float' or entry_type == 'Numeric':
                     sampdict[fmt] = mapper(float, vals)
-                elif sampdict[fmt] == './.' and self.aggro:
-                    sampdict[fmt] = None
-
-            sampdict['name'] = name
-            samp_data.append(sampdict)
-
-        return samp_data
+                else:
+                    sampdict[fmt] = vals
+        return sampdict
 
     def next(self):
         '''Return the next record in the file.'''
@@ -446,12 +654,13 @@ class Reader(object):
             fmt = row[8]
         except IndexError:
             fmt = None
-            samples = None
-        else:
-            samples = self._parse_samples(row[9:], fmt)
 
-        record = _Record(chrom, pos, ID, ref, alt, qual, filt, info, fmt,
-                         samples, self._sample_indexes)
+        record = _Record(chrom, pos, ID, ref, alt, qual, filt, info, fmt, self._sample_indexes)
+
+        if fmt is not None:
+            samples = self._parse_samples(row[9:], fmt, record)
+            record.samples = samples
+
         return record
 
     def fetch(self, chrom, start, end):
@@ -467,6 +676,7 @@ class Reader(object):
         self.reader = self._tabix.fetch(chrom, start, end)
         return self
 
+
 class Writer(object):
 
     fixed_fields = "#CHROM POS ID REF ALT QUAL FILTER INFO FORMAT".split()
@@ -524,9 +734,9 @@ class Writer(object):
         return ';'.join(["%s=%s" % (x, self._stringify(y)) for x, y in info.items()])
 
     def _format_sample(self, fmt, sample):
-        if sample["GT"] == "./.":
+        if sample.data["GT"] == "./.":
             return "./."
-        return ':'.join((str(self._stringify(sample[f])) for f in fmt.split(':')))
+        return ':'.join((str(self._stringify(sample.data[f])) for f in fmt.split(':')))
 
     def _stringify(self, x):
         if type(x) == type([]):
